package com.holub.string;

public class Align
{
	/*****************************************************************
	 * Aligns the text so that the first instance of the align_on charcter
	 * is positioned at align_column, with the pad_character added
	 * to both the left and right of the string to make it work.
	 * If the align_on character isn't found in the input string, then
	 * the output is right ajusted at column alignment_column - 1.
	 * For example:
	 * <pre>
	 *											   align_column
	 *												      |
	 *												      V
	 *													01234
	 *	align( "1.2"  , 5, 2, '.', '_');	// returns	_1.2_	
	 *	align( ".23"  , 5, 2, '.', '_');	// returns	__.23	
	 *	align( "12.3" , 5, 2, '.', '_');	// returns	12.3_
	 *	align( "2.34" , 5, 2, '.', '_');	// returns	_2.34	
	 *	align( "12.34", 5, 2, '.', '_');	// returns	12.34	
	 *	align( "1",     5, 2, '.', '_');	// returns	_1___ 	
	 * </pre>
	 *
	 * @param input 		the String to align
	 * @param column_width	the width of the output string
	 * @param align_column	the column at which the align_on character
	 *						is to be positioned.
	 * @param align_on		the left-most instance of this character in
	 *						the input string is aligned at this column.
	 *						If Align.LAST, then the rightmost character in the
	 *						string is used. If Align.FIRST, then the leftmost
	 *						character in the string is used.
	 * @param pad_character	the chracter to use for padding
	 * @return the input string, with left and right padding added as needed.
	 */

	 static public int FIRST = 0;
	 static public int LAST  = -1;

	 static public String align( String input, int  column_width
											 , int  align_column
											 , int  align_on
											 , char pad_character )
	 {
		int align_character_at = (align_on == FIRST) ? 0 					  :
								 (align_on == LAST ) ? input.length()-1		  :
												       input.indexOf(align_on);
		int left_padding  = align_character_at >= 0
							? Math.max( 0, align_column - align_character_at )
							: Math.max( 0, align_column - input.length() )
							;

		int right_padding = column_width - left_padding - input.length();

		return do_alignment( left_padding, input, right_padding, pad_character );
	}

	/**********************************************************************
	 * Build the output string with the required padding.
	 */
	static private String do_alignment(int left_padding, String input,
									int right_padding, char pad_character)
	{
		StringBuffer work = new StringBuffer();

		while( --left_padding >= 0 )
			work.append( pad_character );

		work.append( input );

		while( --right_padding >= 0 )
			work.append( pad_character );

		return work.toString();
	 }

	/**********************************************************************
	 * Convenience method -- pads with spaces, left-adjusting the text.
	 */
	static public String left( String input, int column_width )
	{	return align( input, column_width, 0, FIRST, ' ' );
	}

	/**
	 * Convenience method -- pads with spaces, right-adjusting the text.
	 */
	static public String right( String input, int column_width )
	{	return align( input, column_width, column_width-1, LAST, ' ' );
	}

	/**
	 * Pads the input string with the pad_character so that it
	 * is column_width charcters wide and centered in the column.
	 * If it can't be centered exactly, the extra padding is placed
	 * on the left.  For example, if the input string is <code>"abc"</code> and
	 * you center it in a 6-character-wide column with <code>'.'</code> as the
	 * pad_character, then the output string is <code>"..abc."</code>.
	 */
	static public String center( String input, int column_width, char pad_character )
	{	int need = column_width - input.length();
		return ( need > 0 )
				? do_alignment( need - (need/2), input, need/2, pad_character )
				: input
				;
	}

	/** Convenience method, pads with spaces.
	*/

	static public String center( String input, int column_width )
	{	return center( input, column_width, ' ' );
	}
	  

	private static class Test
	{	
		public static void main(String[] args)
		{
		com.holub.tools.Tester t = 
				new com.holub.tools.Tester( args.length > 0,
											com.holub.io.Std.out() );

 		t.check( "align.1", "+1.2+", align( "1.2"  , 5, 2, '.', '+') );
 		t.check( "align.2", "++.23", align( ".23"  , 5, 2, '.', '+') );
 		t.check( "align.3", "12.3+", align( "12.3" , 5, 2, '.', '+') );
 		t.check( "align.4", "+2.34", align( "2.34" , 5, 2, '.', '+') );
 		t.check( "align.5", "12.34", align( "12.34", 5, 2, '.', '+') );
 		t.check( "align.6", "+1+++", align( "1",     5, 2, '.', '+') );	
 		t.check( "align.7", "+1.++", align( "1.",    5, 2, '.', '+') );	

		t.check( "align.8", "01234", left ( "01234",	5 		)	);
		t.check( "align.9", "     ", left ( "",	    	5 		)	);
		t.check( "align.a", "X    ", left ( "X", 		5 		)	);

		t.check( "align.b", "01234", right( "01234",	5 		)	);
		t.check( "align.c", "    X", right( "X", 		5 		)	);

		t.check( "align.d", "  X  ", center("X", 		5 		)	);
		t.check( "align.e", "  XX ", center("XX", 		5 		)	);
		t.check( "align.e", "XXXXX", center("XXXXX",	5 		)	);
		t.exit();
		}
	}
}
